@@ -1406,6 +1406,7 @@ seek_slider_pressed_cb (GtkWidget *widget, GdkEventButton *event, Totem *totem)
 {
 	totem->seek_lock = TRUE;
 	totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), TRUE);
+	totem_time_label_set_seeking (TOTEM_TIME_LABEL (totem->tcw_time_label), TRUE);
 
 	if (bacon_video_widget_can_direct_seek (totem->bvw) != FALSE)
 	{
@@ -1433,6 +1434,9 @@ seek_slider_changed_cb (GtkAdjustment *adj, Totem *totem)
 	time = bacon_video_widget_get_stream_length (totem->bvw);
 	totem_statusbar_set_time_and_length (TOTEM_STATUSBAR (totem->statusbar),
 			(int) (pos * time / 1000), time / 1000);
+	totem_time_label_set_time
+			(TOTEM_TIME_LABEL (totem->tcw_time_label),
+			 (int) (pos * time), time);
 
 	if (bacon_video_widget_can_direct_seek (totem->bvw) != FALSE)
 		totem_action_seek (totem, pos);
@@ -1465,6 +1469,8 @@ seek_slider_released_cb (GtkWidget *widget, GdkEventButton *event, Totem *totem)
 	}
 
 	totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), FALSE);
+	totem_time_label_set_seeking (TOTEM_TIME_LABEL (totem->tcw_time_label),
+			FALSE);
 	return FALSE;
 }
 
@@ -2743,6 +2749,12 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 		totem_action_volume_relative (totem, VOLUME_DOWN_OFFSET);
 		break;
 	case GDK_0:
+		if (event->state & GDK_CONTROL_MASK) {
+			totem_action_zoom_reset (totem);
+		} else {
+			totem_action_set_scale_ratio (totem, 0.5);
+		}
+		break;
 	case GDK_onehalf:
 		totem_action_set_scale_ratio (totem, 0.5);
 		break;
@@ -2768,6 +2780,20 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 
 		totem_action_menu_popup (totem, 0);
 		break;
+	case GDK_plus:
+	case GDK_KP_Add:
+		if (!(event->state & GDK_CONTROL_MASK))
+			return FALSE;
+
+		totem_action_zoom_relative (totem, ZOOM_IN_OFFSET);
+		break;
+	case GDK_minus:
+	case GDK_KP_Subtract:
+		if (!(event->state & GDK_CONTROL_MASK))
+			return FALSE;
+
+		totem_action_zoom_relative (totem, ZOOM_OUT_OFFSET);
+		break;
 	default:
 		retval = FALSE;
 	}
@@ -2838,6 +2864,11 @@ on_window_key_press_event (GtkWidget *win, GdkEventKey *event, Totem *totem)
 		case GDK_s:
 		case GDK_Right:
 		case GDK_Left:
+		case GDK_plus:
+		case GDK_KP_Add:
+		case GDK_minus:
+		case GDK_KP_Subtract:
+		case GDK_0:
 			if (event->type == GDK_KEY_PRESS) {
 				return totem_action_handle_key_press (totem, event);
 			}
